<?php

/* 
    Plugin Name: bz2url 
    Plugin URI: http://www.saturn.in/wordpress/
    Version: v1.00
    Author: Jiju Thomas Mathew <jijutm@saturn.in>
    Description: Plugin to create short url and redirection internally
     
    Copyright 2010  Saturn SPL, Technopark
*/

global $bz2_db_version;
$bz2_db_version = "1.1";

/**
 * class bz2url
 */
class bz2url
{

    function __construct(){
        // make sure the activation will invoke our install function
        register_activation_hook(__FILE__, array('bz2url','install'));
        // make sure the url is parsed and redirected if it matches our 
        // short pattern
        add_action( 'init', array('bz2url','tryRedir'));
    }

    // make sure we will be ready with the table
    public static function install(){
        global $wpdb;
        global $bz2_db_version;
        $table_name = $wpdb->prefix . "bz2index";

        $sql = "CREATE TABLE $table_name (
            id char(2),
            url text NULL,
            crc bigint unsigned,
            UNIQUE KEY id (id),
            KEY iCrc (crc) 
        );";
        
        /*
          Some insights:
            - with 2 char the unique options are 2^62 = 4611686018427387904
            - keeping a crc32 of the permalink to make sure permalinks are 
              not duplicated and two short url will point to the same permalink
            - scanning index of the crc32 would be much faster compared to the
              url. Though for any medium portal a partial index on the url would
              be enough.
         */

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);

        add_option("bz2_db_version", $bz2_db_version);    
    } 
    
    // compress a passed url.. we operate only on local urls
    function compress($url){
        global $wpdb;
        $table_name = $wpdb->prefix . "bz2index";
        $short = $wpdb->get_var("SELECT id FROM $table_name WHERE crc=crc32('$url') and url='$url'");
        if(!$short){
            while($short = $this->bz2in()){
               $check = $wpdb->get_var("SELECT count(*) FROM $table_name WHERE id='$short'");
               if($check == 0){
                  $wpdb->query("INSERT INTO $table_name (id, url, crc) VALUES ('$short', '$url', crc32('$url'))");
                  break;
               }
            }
        }
        return site_url() .'/'. $short;
    }
    
    // the real tokenizer
    function bz2in(){
       $a = array_merge(range("a", "z") , range(0, 9), range("A","Z") );
       shuffle($a);
       // with 2 char the unique options are 2^62 = 4611686018427387904
       return substr(join('',$a), 0, 2);
    }

    // this is invoked with the wp->init event, and will redirect if we find
    // a match for /xx in the request url. We handle a premanent redirect to 
    // transfer the page rank to the final target.
    public static function tryRedir(){
      if(preg_match("#^/([a-zA-Z0-9]{2})$#", $_SERVER['REQUEST_URI'], $m)){
        global $wpdb;
        $table_name = $wpdb->prefix . "bz2index";
        $long = $wpdb->get_var(sprintf("SELECT url FROM $table_name WHERE id='%s'", $m[1]), 0, 0);
        if($long){
           header("Location: " . site_url() . $long);
           exit();
        } 
      }
    }    
}

global $bz2;
$bz2 = new bz2url();   

function get_the_shortUrl(){
   global $bz2;
   $url = str_replace(site_url(),'',get_permalink());
   return $bz2->compress($url);
}

function the_shortUrl(){
   echo get_the_shortUrl();
}

